/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   Please note: The implementation of this module is partially based on
*   examples that are provided within the STM32 cube firmware. In case you want
*   to adapt this module to your custom specific hardware, please adapt the
*   hardware initialization code according your needs or integrate the generated
*   initialization code created by using the tool CubeMX.
*   This template provides access to some LEDs and buttons of the board.
*
*******************************************************************************/

#include "ewconfig.h"
#include "stm32l4xx_hal.h"
#include "stm32l4r9i_discovery.h"
#include "stm32l4r9i_discovery_io.h"

#include "ew_bsp_inout.h"

uint8_t BSP_JOY_Init_( void );

#define EW_LED                LED1

static TButtonCallback        ButtonCallback = NULL;
static JOYState_TypeDef       JoyState       = JOY_NONE;


/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitButton
*
* DESCRIPTION:
*   Configures one hardware button of the board used for demo applications.
*
* ARGUMENTS:
*   aButtonCallback - The button callback.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitButton( TButtonCallback aButtonCallback )
{
  ButtonCallback = aButtonCallback;
  BSP_JOY_Init_();
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitLed
*
* DESCRIPTION:
*   Configures one LED of the board used for demo applications.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitLed( void )
{
  BSP_LED_Init( EW_LED );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOn
*
* DESCRIPTION:
*   Switch LED on (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOn( void )
{
  BSP_LED_On( EW_LED );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOff
*
* DESCRIPTION:
*   Switch LED off (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOff( void )
{
  BSP_LED_Off( EW_LED );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutEventHandler
*
* DESCRIPTION:
*   The function EwBspInOutEventHandler is called from the touch screen driver
*   in case of an input event.
*
* ARGUMENTS:
*   aEventId - An optional target specific event ID.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void EwBspInOutEventHandler( int aEventId )
{
  uint16_t GPIO_Pin = ( uint16_t ) aEventId;
  static int UserButtonPressed = 0;


  if ( GPIO_Pin == SEL_JOY_PIN )
  {
    JoyState = BSP_JOY_GetState();

    if ( JoyState != JOY_NONE )
    {
      if ( ! UserButtonPressed )
      {
        UserButtonPressed = 1;
        if ( ButtonCallback )
          ButtonCallback( 1 );
      }
    }
    else if ( UserButtonPressed )
    {
      if ( ButtonCallback )
        ButtonCallback( 0 );
      UserButtonPressed = 0;
    }
  }
}


/*******************************************************************************
 Workaround:
 The original BSP_JOY_Init() has to be replaced, since joystick interrupts
 are needed at rising AND falling edge to detect, if the joystick is released.
*******************************************************************************/

/**
  * @brief  Configures the SEL button of the joystick in EXTI modes.
  * @retval HAL_OK: if all initializations are OK. Other value if error.
  */

uint8_t BSP_JOY_Init_( void )
{
  uint8_t Status = HAL_OK;
  GPIO_InitTypeDef GPIO_InitStruct;

  /* Common configuration for GPIO used for joystick SEL */
  SEL_JOY_GPIO_CLK_ENABLE();
  GPIO_InitStruct.Pin   = SEL_JOY_PIN;
  GPIO_InitStruct.Pull  = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_LOW;

  /* Configure GPIO used for joystick SEL as input with External interrupt */
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  HAL_GPIO_Init(SEL_JOY_GPIO_PORT, &GPIO_InitStruct);
  /* Enable and set corresponding EXTI Interrupt to the lowest priority */
  HAL_NVIC_SetPriority((IRQn_Type)(SEL_JOY_EXTI_IRQn), 0x08, 0x00);
  HAL_NVIC_EnableIRQ((IRQn_Type)(SEL_JOY_EXTI_IRQn));

  return Status;
}


/* msy */
